dofile 'Class.lua'
dofile 'linkedlist.lua'

-- Components are objects that handle messages to perform certain tasks. They are simply tables that map functions
-- and these functions are called through the object's handleMessage function.

-- handleMessage iterates through all components, and checks to see if a given component has an entry named the same as the passed message string.
-- If so, then that function is called with the owner object and the given args. For example:
-- The "AddFrameTime" message is intended to add elapsed_time to animations and to use percent_within_tick to update visual positions of logical
-- objects. So a component that wishes to handle this must implement a function called AddFrameTime that accepts the component's owner-object,
-- plus a table with the elapsed and percent values:
-- component={}
-- function component.AddFrameTime(self, owner, args)
--   owner:handleMessage("StepAnimation", {elapsed=args.elapsed})
--   owner:handleMessage("UpdateVisualLocation", {percent=args.percent})
-- end
-- Component message handlers are passed the owner object so that they can access data, and pass the owner off to other messages as needed.


Object=class(function(a, guid) a.components=LinkedList() a.guid=guid end)

function Object.addComponent(self,comp)
	self.components:push_front(comp)
end

function Object.removeComponent(self, comp)
	self.components:remove(comp)
end

function Object.handleMessage(self, msg, args)
	local p=self.components.list
	local k
	while p~=nil do
		k=p.value
		if k[msg]~=nil then
			k[msg](k,self,args)
			
		end
		p=p.next
	end
end

function Object.handleRequest(self,msg,args)
	local p=self.components.list
	local k
	local retvalue=LinkedList()
	while p~=nil do
		k=p.value
		if k[msg]~=nil then
			retvalue:push_front(k[msg](k,self,args))
		end
		p=p.next
	end
	return retvalue
end