require 'Class'
require 'linkedlist'
require 'hotkeytable'

GUIBase = class(function(a,x,y,w,h) a.x=x a.y=y a.w=w a.h=h a.children=LinkedList() a.shown=false a.hotkeys=HotkeyActionTable() end)

function GUIBase:setParent(obj)
	self.parent=obj
end

function GUIBase:addHotkeyMapping(key,func, func_obj)
	self.hotkeys:insertMapping(key,func,func_obj)
end

function GUIBase:removeHotkeyMapping(key)
	self.hotkeys:removeMapping(key)
end

function GUIBase:setPosition(x,y)
	self.x=x
	self.y=y
end


function GUIBase:insertChildBack(child)
  --child.parent=self
  --table.insert(self.children,1,child)
  self.children:push_back(child)
  child:setParent(self)
end

function GUIBase:insertChildFront(child)
  --child.parent=self
  --table.insert(self.children, child)
  self.children:push_front(child)
  child:setParent(self)
end

function GUIBase:removeChild(child)
	self.children:remove(child)
	child.parent=nil
end

function GUIBase:drawChildren(t)
  local node=self.children.list
  while node~=nil do
	node.value:draw(t)
	node=node.next
  end
end

function GUIBase:show()
  self.shown=true
  self:showChildren()
end

function GUIBase:hide()
  self.shown=false
  self:hideChildren()
end

function GUIBase:toggle()
	if self:isShown() then self:hide()
	else self:show()
	end
end

function GUIBase:drawSelf(t)
end

function GUIBase:draw(t)
	if(self:isShown()==false) then return end
	self:drawSelf(t)
	self:drawChildren(t)
end

function GUIBase:showChildren()
	local node=self.children.list
	while node ~= nil do
		node.value:show()
		node=node.next
	end
 

end

function GUIBase:hideChildren()
	local node=self.children.list
	while node~=nil do
		node.value:hide()
		node=node.next
	end
		

end

function GUIBase:isShown()
  return self.shown
end

function GUIBase:getScreenLocation()
  local px,py = self:getParentScreenLocation()
  return px+self.x, py+self.y
end

function GUIBase:getParentScreenLocation()
  if(self.parent) then
    return self.parent:getScreenLocation()
  else
    return 0,0
  end
end

function GUIBase:checkPoint(x,y)
  local px,py = self:getScreenLocation()
  if x<px or y<py or x>px+self.w or y>py+self.h then
    return false
  else
    return true
  end
end

function GUIBase:mouseOverSelf(x,y)
	return self:checkPoint(x,y)
end

function GUIBase:mouseOver(x,y)
	if self:mouseOverSelf(x,y) or self:mouseOverChildren(x,y) then return true
	else return false
	end
end

function GUIBase:mouseOverChildren(x,y)
	local node=self.children.list
	while node~=nil do
		if node.value.checkPoint(x,y) then return true end
		node=node.next
	end
	return false
end

function GUIBase:handleEventChildren(se)
	local node=self.children.list
	while node~= nil do
		if node.value:handleEvent(se) then return true end
		node=node.next
	end
	return false
end

function GUIBase:handleEventSelf(se)
	return false
end

function GUIBase:handleEvent(se)
	if self:isShown()==false then return false end
	if self:handleEventChildren(se)==false then return self:handleEventSelf(se) end
end


  