
islandtree=
{
	-- The main shape of the continent is founded on a fuzzy disk
	{name="ContinentSphere", type="sphere", cx=0.5, cy=0.5, radius=0.5*0.75},
	{name="ContinentXTurb", type="fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		frequency=3, num_octaves=10},
	{name="ContinentYTurb", type="fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		frequency=3, num_octaves=10},
	{name="ContinentTurb", type="turbulence", main_source="ContinentSphere", x_axis_source="ContinentXTurb", y_axis_source="ContinentYTurb",
		x_power=0.2, y_power=0.2},
	
	{name="ContinentDisk", type="greyscale_implicitadapter", source="ContinentTurb", seamlessmode=ESM_NONE},
	
	-- Create a selection mask to outline the shape of the continent and blend between ocean floor and continent landforms.
	{name="GConstant0", type="greyscale_constant", constant=0},
	{name="GConstant1", type="greyscale_constant", constant=1},
	{name="ContinentMask", type="greyscale_select", main_source="ContinentDisk", low_source="GConstant0", high_source="GConstant1",
		threshold=0.3, falloff=0.15},
		
	-- Create a selection mask to define the shoreline
	{name="ShorelineMask", type="greyscale_select", main_source="ContinentDisk", low_source="GConstant0", high_source="GConstant1",
		threshold=0.15, falloff=0.01},
	
		
	-- Create a fractal for the ocean floor
	{name="OceanFloorFractal", type="greyscale_fractal", fractal_type="HYBRIDMULTI", basis_type="GRADIENT", interp_type="QUINTIC",
		num_octaves=8, frequency=6, remaprange=true, low=0, high=0.3},
	
	
	-- Create a fractal for mountains
	{name="MountainFractal", type="greyscale_fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		num_octaves=10, frequency=22, remaprange=true, low=0.45,high=1},
		
	-- Create a fractal for rolling hills
	{name="RollingHillsFractal", type="greyscale_fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		num_octaves=8, frequency=6, remaprange=true, low=0.25, high=0.45},
		
	-- Create a fractal mask to separate areas of mountain from areas of non-mountain
	{name="MountainMaskFractal", type="greyscale_fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		num_octaves=8, frequency=6, remaprange=true, low=0, high=1},
	{name="MountainMask", type="greyscale_select", main_source="MountainMaskFractal", low_source="GConstant0", high_source="GConstant1",
		threshold=0.7, falloff=0.19},
		
	-- Create a blender to combine mountains with lower terrain
	{name="MountainLowlandBlend", type="greyscale_blend", main_source="MountainMask", low_source="RollingHillsFractal", high_source="MountainFractal"},
	
	
	-- Continent blend
	{name="ContinentBlend", type="greyscale_blend", main_source="ContinentMask", low_source="OceanFloorFractal", high_source="MountainLowlandBlend"},
		
		
	{name="ContinentBump", type="greyscale_bumpmap", source="ContinentBlend", sharpness=4, light={1.5,3.5,-1.5}},
	
	
	
	-- RGBA Modules
	{name="RGBAConstantBlueDark", type="rgba_constant", red=0.2, green=0.5, blue=0.9},
	{name="RGBAConstantBlueLight", type="rgba_constant", red=0.5, green=0.6, blue=1},
	{name="RGBAConstantGrey", type="rgba_constant", red=0.5, green=0.5, blue=0.5},
	{name="RGBAConstantWhite", type="rgba_constant", red=1,green=1,blue=1},
	{name="RGBAConstantBrown", type="rgba_constant", red=195/255, green=130/255, blue=46/255},
	{name="RGBAConstantGreen", type="rgba_constant", red=0.2, green=0.7, blue=0.1},
	{name="RGBAConstantDarkGreen", type="rgba_constant", red=0.1, green=0.5, blue=0.2},
	
	-- Create a stretched ridged multi fractal to setup a wave pattern
	{name="WaveRippleFractal", type="greyscale_fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		num_octaves=3, frequency=12, mapx1=0, mapy1=0, mapx2=1, mapy2=5, remaprange=true, low=0, high=1},
	{name="WaveRippleBump", type="greyscale_bumpmap", source="WaveRippleFractal", sharpness=0.1, light={1.5,3.5,-1.5}},
	{name="WaveLightColor", type="rgba_combinealpha", greyscale_source="WaveRippleFractal", rgba_source="RGBAConstantBlueLight"},
	{name="WaveColor", type="rgba_blend", source1="WaveLightColor", source2="RGBAConstantBlueDark", blendop1=0, blendop2=2},
	
	{name="RGBAWavePattern", type="rgba_multrgbagreyscale", greyscale_source="WaveRippleBump", rgba_source="WaveColor"},
	
	-- Multiply the mountain elevation map by the mountain mask and the shoreline mask in order to get a "real" elevation map for
	-- adding snow to higher elevation mountains,, then set up a selection buffer and do a blend between grey and white
	{name="MountainFractalMultiply", type="greyscale_combiner", combiner_type=ECT_MULT, source_0="MountainFractal", source_1="MountainMask",
		source_2="ContinentMask"},
		
	{name="MountainSnowLineSelect", type="greyscale_select", main_source="MountainFractalMultiply", low_source="GConstant0", high_source="GConstant1",
		threshold=0.45, falloff=0.075},
		
	{name="MountainSnowLayer", type="rgba_combinealpha", greyscale_source="MountainSnowLineSelect", rgba_source="RGBAConstantWhite"},
	{name="MountainSnowBlend", type="rgba_blend", source1="MountainSnowLayer", source2="RGBAConstantGrey", blendop1=0, blendop2=2},
	
	-- Setup a fractal to blend between green and brown for lowland colors
	{name="LowlandBlendMask", type="greyscale_fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		num_octaves=6, frequency=4, remaprange=true, low=0, high=1},
	{name="LowlandColorLayer", type="rgba_combinealpha", greyscale_source="LowlandBlendMask", rgba_source="RGBAConstantGreen"},
	{name="LowlandColorBlend", type="rgba_blend", source1="LowlandColorLayer", source2="RGBAConstantBrown", blendop1=0, blendop2=2},
	
	-- Setup a tighter version of the mountain mask
	{name="MountainColorMask", type="greyscale_select", main_source="MountainMaskFractal", low_source="GConstant0", high_source="GConstant1",
		threshold=0.5, falloff=0.15},
		
	-- Invert the mountain color mask, set up a forest fractal mask and multiply it by the mountain mask, ie no forest cover in mountains
	--{name="MountainColorMaskInvert", type="greyscale_remaprange", source="MountainColorMask", low=1, high=0},
	{name="MountainColorMaskInvert", type="greyscale_scaleoffset", source="MountainColorMask", scale=-1, offset=1},
	{name="ForestBlendMask", type="greyscale_fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		num_octaves=6, frequency=6, remaprange=true, low=0, high=1},
	{name="ForestBlendMaskSelect", type="greyscale_select", main_source="ForestBlendMask", low_source="GConstant0", high_source="GConstant1",
		threshold=0.5, falloff=0.05},
	{name="ForestMask", type="greyscale_combiner", combiner_type=ECT_MULT, source_0="ForestBlendMaskSelect", source_1="MountainColorMaskInvert"},
	
	-- Setup a color blend layer for forest/lowland
	-- First, set up a bump source to make bumpy forest color
	{name="ForestBumpFractal", type="greyscale_fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		num_octaves=8, frequency=64, remaprange=true, low=0, high=1},
	{name="ForestBump", type="greyscale_bumpmap", source="ForestBumpFractal", sharpness=1, light={1.5,3.5,-1.5}},
	{name="ForestColorBump", type="rgba_multrgbagreyscale", greyscale_source="ForestBump", rgba_source="RGBAConstantDarkGreen"},
	
	-- Now, blend in forest color
	{name="ForestColorLayer", type="rgba_combinealpha", greyscale_source="ForestMask", rgba_source="ForestColorBump"},
	{name="VegetationColorBlend", type="rgba_blend", source1="ForestColorLayer", source2="LowlandColorBlend", blendop1=0, blendop2=2},
	
	{name="RGBAMountainLayer", type="rgba_combinealpha", greyscale_source="MountainColorMask", rgba_source="MountainSnowBlend"},
	{name="RGBAMountainBlend", type="rgba_blend", source1="RGBAMountainLayer", source2="VegetationColorBlend", blendop1=0, blendop2=2},
	
	
	-- Blend between land colors and ocean colors using the shoreline mask
	{name="RGBALandMap", type="rgba_combinealpha", greyscale_source="ShorelineMask", rgba_source="RGBAMountainBlend"},
	{name="RGBACombineLandOcean", type="rgba_blend", source1="RGBALandMap", source2="RGBAWavePattern", blendop1=0, blendop2=2},
	
	-- Multiple the whole thing by the elevation bump-map
	{name="RGBACombineBump", type="rgba_multrgbagreyscale", greyscale_source="ContinentBump", rgba_source="RGBACombineLandOcean"},
	
	{name="RGBAAdapter", type="rgba_greyscaleadapter", source="MountainFractalMultiply"},
	{name="Output", type="output", source="RGBACombineBump", mode=0, width=256, height=256},
	
	
	
	
	-- Connectors for building tile maps
	
	-- Ocean/Land
	{name="OceanLandMapper", type="greyscale_select", main_source="ShorelineMask", low_source="GConstant0", high_source="GConstant1",
		threshold=0.5, falloff=0},
	
	-- Grass
	{name="GrassLayer", type="greyscale_select", main_source="LowlandBlendMask", low_source="GConstant0", high_source="GConstant1",
		threshold=0.6, falloff=0},
	{name="GrassMapper", type="greyscale_combiner", combiner_type=ECT_MULT, source_0="GrassLayer", source_1="OceanLandMapper"},
		
	-- Mountain Mask
	{name="MountainLayer", type="greyscale_select", main_source="MountainMaskFractal", low_source="GConstant0", high_source="GConstant1",
		threshold=0.5, falloff=0},
	{name="MountainMapper", type="greyscale_combiner", combiner_type=ECT_MULT, source_0="MountainLayer", source_1="OceanLandMapper"},
	{name="InverseMountainMapper", type="greyscale_scaleoffset", source="MountainMapper", scale=-1, offset=1},
		
	-- Mountain Steepness
	{name="MountainSteep", type="greyscale_bumpmap", source="MountainFractal", sharpness=2, light={0,1,0}},
	{name="MountainSteepLayer", type="greyscale_select", main_source="MountainFractal", low_source="GConstant1", high_source="GConstant0",
		threshold=0.65, falloff=0},
	{name="MountainSteepMapper", type="greyscale_combiner", combiner_type=ECT_MULT, source_0="MountainSteepLayer", source_1="MountainMapper"},
		
	-- Snow
	{name="SnowLayer", type="greyscale_select", main_source="MountainFractalMultiply", low_source="GConstant0", high_source="GConstant1",
		threshold=0.25, falloff=0},
	{name="SnowMapper", type="greyscale_combiner", combiner_type=ECT_MULT, source_0="SnowLayer", source_1="MountainMapper"},
	
	-- Forest
	{name="ForestLayer", type="greyscale_select", main_source="ForestMask", low_source="GConstant0", high_source="GConstant1",
		threshold=0.5, falloff=0},
	{name="ForestMapper", type="greyscale_combiner", combiner_type=ECT_MULT, source_0="ForestLayer", source_1="OceanLandMapper",
		source_2="InverseMountainMapper"}
	
}

function setIslandMappingRange(x1,y1,x2,y2)
	-- Get implicit adapters for the following:
	local i={}
	i[1]=island:getFunction("ContinentDisk")
	i[2]=island:getFunction("OceanFloorFractal")
	i[3]=island:getFunction("MountainFractal")
	i[4]=island:getFunction("RollingHillsFractal")
	i[5]=island:getFunction("MountainMaskFractal")
	i[6]=island:getFunction("WaveRippleFractal")
	i[7]=island:getFunction("LowlandBlendMask")
	i[8]=island:getFunction("ForestBlendMask")
	i[9]=island:getFunction("ForestBumpFractal")
	
	for c=1,#i,1 do
		i[c]:setXMapRange(x1,x2)
		i[c]:setYMapRange(y1,y2)
	end
	
end

function dumpTileMaps()
	local rgba=CRGBAAdapterGreyscale()
	local out=COutputModule(256,256, EGT_RGB)
	out:setRGBASource(rgba)
	
	local f=island:getFunction("OceanLandMapper")
	rgba:setGreyscaleSource(f)
	out:eval("land.tga")
	
	f=island:getFunction("GrassMapper")
	rgba:setGreyscaleSource(f)
	out:eval("grass.tga")
	
	f=island:getFunction("MountainMapper")
	rgba:setGreyscaleSource(f)
	out:eval("mountain.tga")
	
	f=island:getFunction("MountainSteepMapper")
	rgba:setGreyscaleSource(f)
	out:eval("steep.tga")
	
	f=island:getFunction("MountainSteep")
	rgba:setGreyscaleSource(f)
	out:eval("steepness.tga")
	
	f=island:getFunction("SnowMapper")
	rgba:setGreyscaleSource(f)
	out:eval("snow.tga")
	
	f=island:getFunction("ForestMapper")
	rgba:setGreyscaleSource(f)
	out:eval("forest.tga")
end

function get_tile_maps(dim)
	
	local f=island:getFunction("OceanLandMapper")
	local land=CArray2Dd()
	land:init(dim,dim)
	f:map(land,1)
	
	f=island:getFunction("GrassMapper")
	local grass=CArray2Dd()
	grass:init(dim,dim)
	f:map(grass,1)
	
	f=island:getFunction("MountainMapper")
	local mountain=CArray2Dd()
	mountain:init(dim,dim)
	f:map(mountain,1)
	
	f=island:getFunction("MountainSteepMapper")
	local steep=CArray2Dd()
	steep:init(dim,dim)
	f:map(steep,1)
	
	f=island:getFunction("SnowMapper")
	local snow=CArray2Dd()
	snow:init(dim,dim)
	f:map(snow,1)
	
	f=island:getFunction("ForestMapper")
	local forest=CArray2Dd()
	forest:init(dim,dim)
	f:map(forest,1)
	
	return land,grass,mountain,steep,snow,forest
end
