-- Isometric World Manager

require 'Class'
require 'baseobject'
require 'levelgenerator'

IsometricWorld = class(function(a, pixelscale, worldsize)
	a.pixelscale=pixelscale
	a.worldsize=worldsize
	a.view=CIsoSceneView(pixelscale,2)
	a.view:setCameraCenter(0,0)
	a.generator=LevelGenerator(worldsize)
	
	a.nextID=1
	a.objects={}
	end)
	
function IsometricWorld:reverseProjectOnPlane(mousex, mousey, planey)
	return self.view:reverseProjectOnPlane(mousex, mousey, planey)
end

function IsometricWorld:setCameraCenter(curx, cury)
	self.view:setCameraCenter(curx, cury)
end

function IsometricWorld:render()
	self.view:prepForRender()
	if self.scene~=nil then self.view:renderChunk(self.scene) end
	self.view:render()
end

function IsometricWorld:createObject()
	local o=Object(self.nextID)
	self.objects[self.nextID]=o
	self.nextID = self.nextID+1
	return o
end

function IsometricWorld:getObject(guid)
	return self.objects[guid]
end

function IsometricWorld:killObject(o)
	local guid=o.guid
	self.objects[guid]:handleMessage("KillObject")
	self.objects[guid]=nil
end

function IsometricWorld:AddFrameTime(elapsedtime, percentwithintick)
	-- AddFrameTime to all active objects, starting with playerobject if set
	local msg={elapsed_time=elapsedtime, percent_within_tick=percentwithintick}
	
	-- Now, iterate object list
	local i,j
	for i,j in pairs(self.objects) do
		j:handleMessage("AddFrameTime", msg)
	end
end

function IsometricWorld:UpdateLogic()
	-- Update logic
	local i,j
	for i,j in pairs(self.objects) do
		j:handleMessage("UpdateLogic")
	end
end

function IsometricWorld:generateWorld(randomseed)
	self.scene=CIsoSceneChunk(0,0,self.worldsize,self.worldsize)
	self.scene:setOffset(0,0)
	self.light=CLightChunk(0,0,self.worldsize+1,self.worldsize+1)
	self.light:setAmbient(1,1,1)
	self.light:setOffset(0,0)
	self.scene:setLight(self.light)
	
	self.generator:generate(self.scene, randomseed)
	collectgarbage()
	
	
end

function IsometricWorld:generateRandomWorld()
	local seed=randInt()
	self:generateWorld(seed)
end


