-- levelgenerator.lua
require 'Class'
require 'islandtree'
require 'noisemoduletree'
require 'tiletranslate'

LevelGenerator=class(function(a,worldsize)
	a.worldsize=worldsize
	a.island=NoiseModuleTree(islandtree)
	end)
	
function LevelGenerator:getTileMaps()
	local dim=self.worldsize
	local f=self.island:getFunction("OceanLandMapper")
	local land=CArray2Dd()
	land:init(dim,dim)
	f:map(land,1)
	
	f=self.island:getFunction("GrassMapper")
	local grass=CArray2Dd()
	grass:init(dim,dim)
	f:map(grass,1)
	
	f=self.island:getFunction("MountainMapper")
	local mountain=CArray2Dd()
	mountain:init(dim,dim)
	f:map(mountain,1)
	
	f=self.island:getFunction("MountainSteepMapper")
	local steep=CArray2Dd()
	steep:init(dim,dim)
	f:map(steep,1)
	
	f=self.island:getFunction("SnowMapper")
	local snow=CArray2Dd()
	snow:init(dim,dim)
	f:map(snow,1)
	
	f=self.island:getFunction("ForestMapper")
	local forest=CArray2Dd()
	forest:init(dim,dim)
	f:map(forest,1)
	
	return land,grass,mountain,steep,snow,forest
end

function LevelGenerator:skewMinimap(m)
	-- Skew the minimap into a diamond with a transparent background
	local tw,th=m:getWidth(), m:getHeight()
	
	local buffer=CArray2Drgbaf()
	local w,h = tw, th
	buffer:init(w,h)
  
	buffer:fill(CRGBAf(0,0,0,0))
  
	buffer:rasterTriangleTex(m,nil,CVec4i(1,h/2-1,tw-1,0), CVec4i(w/2+1,-1,0,0), CVec4i(w/2+1,h/2-1,tw/2,th/2),0)
	buffer:rasterTriangleTex(m,nil,CVec4i(w/2-1,-1,0,0), CVec4i(w/2-1,h/2-1,tw/2,th/2), CVec4i(w-2,h/2-1,0,th-1),0)
	buffer:rasterTriangleTex(m,nil,CVec4i(1,h/2,tw-1,0),CVec4i(w/2,h/2,tw/2,th/2),CVec4i(w/2,h-1,tw-1,th-1),0)
	buffer:rasterTriangleTex(m,nil,CVec4i(w/2-1,h/2,tw/2,th/2), CVec4i(w-2,h/2,0,th-1), CVec4i(w/2-1,h,tw-1,th-1),0)
	
	return buffer
end
	
function LevelGenerator:generate(scene,seed)
	self.island:setSeedAll(seed)
	
	local mmsize=256
	
	-- Build mini-map
	local m=CArray2Drgbaf()
	m:init(mmsize,mmsize)
	self.island:getFunction("RGBACombineBump"):map(m,1)
	
	m=self:skewMinimap(m)
	
	saveTGA_RGBAf("minimap.tga", m)
	
	if minimappanel~=nil then
		gui_window:removeChild(minimappanel)
		minimappanel=nil
		collectgarbage()
	end
	
	if mapmarker==nil then
		mapmarker=GUIBackgroundPiece(400,300,8,8,0,0,"mapmarker.tga",1,1)
	else
		gui_window:removeChild(mapmarker)
	end
	
	-- Minimap corners for mapmarker placement
	mm1_x=(mmsize)*0.5
	mm1_y=600-mmsize
	mm2_x=mmsize
	mm2_y=600-(mmsize)*0.5
	mm3_x=0
	mm3_y=600-(mmsize)*0.5
	mm4_x=(mmsize)*0.5
	mm4_y=600
		
	
	minimappanel=GUIBackgroundPiece(0,600-(mmsize),mmsize,mmsize,0,0,"minimap.tga",1,1)
	gui_window:insertChildFront(mapmarker)
	gui_window:insertChildFront(minimappanel)
	minimappanel:show()
	mapmarker:show()
	
	local land,grass,mountain,steep,snow,forest=self:getTileMaps()
	
	-- Populate ground map
	local groundmap=CArray2Dd()
	groundmap:init(self.worldsize, self.worldsize)
	groundmap:fill(0)
	
	local x,y
	for x=0,self.worldsize-1,1 do
		for y=0,self.worldsize-1,1 do
			if land:get(x,y)==1 then groundmap:set(x,y,1) end
			if grass:get(x,y)==1 then groundmap:set(x,y,2) end
		end
	end
	
	-- Build up ground
	for x=0,self.worldsize-1,1 do
		for y=0,self.worldsize-1,1 do
			local base=groundmap:get(x,y)
			if base==0 then
				scene:instanceStaticEntity("data/pieces/ground.msh", "data/tilesets/basictiles/water.tga",1,1,0,0,x,0,y,1,1,1,0)
			elseif base==1 then
				scene:instanceStaticEntity("data/pieces/ground.msh", "data/tilesets/basictiles/dirt0.tga",1,1,0,0,x,0,y,1,1,1,0)
			else
				scene:instanceStaticEntity("data/pieces/ground.msh", "data/tilesets/basictiles/grass0.tga",1,1,0,0,x,0,y,1,1,1,0)
			end
			
			-- Check for dirt transitions
			if base==0 then
				local edge,corner=get_tile_codes(groundmap,1,x,y)
				if edge~=0 then
					scene:instanceStaticEntity("data/pieces/ground.msh", "data/tilesets/basictiles/dirt"..edge..".tga",1,1,0,0,x,0.0,y,1,1,1,1)
				end
				if corner>15 then
					scene:instanceStaticEntity("data/pieces/ground.msh", "data/tilesets/basictiles/dirt"..corner..".tga",1,1,0,0,x,0.0,y,1,1,1,1)
				end
			end
			
			-- Check for grass transitions
			if base==0 or base==1 then
				local edge,corner=get_tile_codes(groundmap,2,x,y)
				if edge~=0 then
					scene:instanceStaticEntity("data/pieces/ground.msh", "data/tilesets/basictiles/grass"..edge..".tga",1,1,0,0,x,0.0,y,1,1,1,1)
				end
				if corner>15 then
					scene:instanceStaticEntity("data/pieces/ground.msh", "data/tilesets/basictiles/grass"..corner..".tga",1,1,0,0,x,0.0,y,1,1,1,1)
				end
			end
			
			if forest:get(x,y)==1 then
				local c
				for c=1,5,1 do
					scene:instanceStaticEntity("data/pieces/billboardfoot.msh", "data/tilesets/testtree"..randRange(1,4)..".tga", 1,1,0,0,x+rand01(), 0.0, y+rand01(), 0.75,0.75,0.75,2)
				end
			end
			
			if mountain:get(x,y)==1 then
				local n
				if snow:get(x,y)==1 then
					n="data/tilesets/mountain"..randRange(1,4)..".tga"
				else
					n="data/tilesets/lowmountain"..randRange(1,4)..".tga"
				end
				scene:instanceStaticEntity("data/pieces/billboardfoot.msh", n, 1, 1, 0, 0, x+1,0,y+1, 1.75,1.75,1.75, 2)
			end
		end
	end
	
end

