-- Logical position component
-- Implement a component that tracks an object's logical position and handles UpdateObjectVisuals to set the visual position

dofile('baseobject.lua')

LogicalPositionComponent=class(function(a) a.xlast,a.ylast,a.zlast=0,0,0 a.xcur,a.ycur,a.zcur=0,0,0 end)

function LogicalPositionComponent.AddFrameTime(self, owner, args)
	-- args contains percent_within_tick of current frame
	local t=args.percent_within_tick
	local xvis=self.xlast + t*(self.xcur-self.xlast)
	local yvis=self.ylast + t*(self.ycur-self.ylast)
	local zvis=self.zlast + t*(self.zcur-self.zlast)
	
	
	owner:handleMessage("UpdateObjectVisualPosition", {x=xvis, y=yvis, z=zvis})
end

function LogicalPositionComponent.SetObjectLogicalPosition(self, owner, args)
	self.xlast=args.x
	self.xcur=args.x
	
	self.ylast=args.y
	self.ycur=args.y
	
	self.zlast=args.z
	self.zcur=args.z
	
end

function LogicalPositionComponent.UpdateObjectLogicalPosition(self, owner, args)
	self.xlast=self.xcur
	self.ylast=self.ycur
	self.zlast=self.zcur
	self.xcur=args.x
	self.ycur=args.y
	self.zcur=args.z
end

function LogicalPositionComponent.UpdateObjectLogicalPositionFix(self,owner,args)
	self.xlast=self.xcur
	self.ylast=self.ycur
	self.zlast=self.zcur
end

function LogicalPositionComponent.UpdateObjectLogicalPositionRelative(self, owner, args)
	-- Update using a movement vector to move relative to current position
	--print("updating log pos rel: "..args.x..","..args.y..","..args.z)
	self.xlast=self.xcur
	self.ylast=self.ycur
	self.zlast=self.zcur
	self.xcur=self.xcur+args.x
	self.ycur=self.ycur+args.y
	self.zcur=self.zcur+args.z
end

function LogicalPositionComponent.GetObjectLogicalPosition(self, owner, args)
	-- Query to get object position
	return {x=self.xcur, y=self.ycur, z=self.zcur}
end

