
dofile("isoworld.lua")
dofile("levelviewercomponent.lua")
dofile("logicalpositioncomponent.lua")
dofile("SFMLtoASCII.lua")

dofile("gui_console.lua")
dofile("gui_window.lua")
dofile("gui_panel.lua")
dofile("gui_button.lua")

oldprint=print
function print(s)
	logString(s)
end
function console_print(s)
	logString("Console Print: "..s)
	if console then
		console.cwin:printString(s)
	end
end

core_kernel=
{
	reset_loop=false,
	running=false,
	oldtime=0,
	logicupdatecounter=0,
	logicstep=0,
	curtime=0,
	framedelta=0,
	percentwithintick=0,
	controlledobject=nil,
	lbuttondown=false,
	rbuttondown=false,
	mbuttondown=false,
}



function core_kernel:kill()
	self.running=false
end

function core_kernel.handleEvent(self, event)
	if(event.Type==event.Closed ) then
		self.running=false
	end
	
	if(gui_window) then gui_window:handleEvent(event) end
	
	if(event.Type==event.MouseButtonPressed and event.MouseButton.Button==sf.Mouse.Left) then
		-- Send controlled object LMBDown, with reverse-projected mouse_x and mouse_y
		
		local vec=world:reverseProjectOnPlane(event.MouseButton.X, event.MouseButton.Y,0)
		levelviewer:handleMessage("LMBDown", {mouse_x=vec.x, mouse_y=vec.z})
		return
	end
	if(event.Type==event.MouseButtonReleased and event.MouseButton.Button==sf.Mouse.Left) then
		levelviewer:handleMessage("LMBUp")
		return
	end
	
		

end

function core_kernel.run(self,updates_per_sec)
	self.reset_loop=false
	self.running=true
	self.oldtime=core_system:getTime()
	self.logicupdatecounter=0
	self.logicstep=1.0/updates_per_sec
	event=sf.Event()
	
	local framecount=0
	local totaltime=0
	
	while(self.running) do
		if(self.reset_loop) then
			self.oldtime=core_system:getTime()
			self.logicupdatecounter=0
			self.resetloop=false;
		end
		
		-- Process input
		while (core_system:getEvent(event)) do
			self:handleEvent(event)
			if self.running==false then return end
			--print("FPS:"..((framecount/totaltime))) return end
		end
		
		self.curtime=core_system:getTime()
		self.framedelta=(self.curtime-self.oldtime)
		totaltime=totaltime+self.framedelta
		
		-- Update
		self.logicupdatecounter = self.logicupdatecounter+self.framedelta
		--local updatecount=0
		while(self.logicupdatecounter >= self.logicstep) do
			world:UpdateLogic()
			self.logicupdatecounter = self.logicupdatecounter - self.logicstep
		end
		
		
		self.percentwithintick = self.logicupdatecounter / self.logicstep
		world:AddFrameTime(self.framedelta, self.percentwithintick)
		core_system:clear()
		world:render()
		
		if(gui_window) then
			gui_window:draw(self.percentwithintick)
		end
		-- Swap buffers
		core_system:render()
		framecount=framecount+1
		
		collectgarbage()
		
		
		-- Done
		self.oldtime=self.curtime
	end
	
end


-- Setup global state and managers
core_system=CIsoCoreSystem(800,600,"Test", false)


gui_window=GUIWindow(0,0,800,600)
gui_window:show()

console=GUIConsole(0,0,800,500)
console:setup("font.glf", 300)
console:hide()

gui_window:addHotkeyMapping(sf.Key.Tilde, console.toggle, console)
gui_window:addHotkeyMapping(sf.Key.Escape, core_kernel.kill,  core_kernel)

gui_window:insertChildFront(console)

-- Buttons
genbutton=GUIButton(800-256,600-32,128,32,"generatebutton.tga")
gui_window:insertChildFront(genbutton)
quitbutton=GUIButton(800-128,600-32,128,32,"quitbutton.tga")
gui_window:insertChildFront(quitbutton)

genbutton:show()
quitbutton:show()



setSeedTime()

world=IsometricWorld(192,64)
world:generateWorld(randInt())

genbutton:setOnClick(world.generateRandomWorld, world)
quitbutton:setOnClick(core_kernel.kill, core_kernel)

levelviewer=world:createObject()
levelviewer:addComponent(LogicalPositionComponent())
levelviewer:addComponent(LevelViewerComponent())

levelviewer:handleMessage("SetObjectLogicalPosition",{x=32, y=0, z=32})
levelviewer:handleMessage("UpdateObjectVisualPosition", {x=32, y=0, z=32})




core_kernel:run(25)

--generator:saveimg()
core_system:closeWindow()
