-- Tile map translation

-- Functions to assist in translating tilemaps and generating transition codes

-- Transitions codes come in 2 "flavors": edge codes and corner codes.
-- An edge code denotes which of the edge tiles a cell is, depending on the neighbors to the top, left, right and bottom,
-- and ranges in the range of 1..15.
-- A corner code is actually a set of 4 flags indicating if a given corner piece should be applied. The generation of this code
-- takes into account the corner neighbors as well as the adjacent neighbors, so that corner pieces are not applied of a "connecting"
-- edge is already applied via an edge code.

-- That is, the top-left corner piece is not applied if the top edge or left edge is applied via an edge code.

-- If a cell is of the given typ, edge-code 0 and no corner-codes are returned, so that the corresponding cell can be set in the tile-map

function get_tile_codes(map, value, x,y)
	-- Check all the adjacent neighbors
	local t,l,r,b,tl,tr,bl,br=false,false,false,false,false,false,false,false
	local mw,mh=map:getWidth(), map:getHeight()
	
	-- First, check if is tile type or higher
	if map:get(x,y)>=value then return 0,0 end
	
	-- Check top
	if y-1>=0 then
		if map:get(x,y-1)>=value then b=true end
	end
	
	-- Check left
	if x-1>=0 then
		if map:get(x-1,y)>=value then l=true end
	end
	
	-- Check right
	if x+1<mw then
		if map:get(x+1,y)>=value then r=true end
	end
	
	-- Check bottom
	if y+1<mh then
		if map:get(x,y+1)>=value then t=true end
	end
	
	-- Corners
	if x-1>=0 and y-1>=0 then
		if map:get(x-1,y-1)>=value then bl=true end
	end
	
	if x+1<mw and y-1>=0 then
		if map:get(x+1,y-1)>=value then br=true end
	end
	
	if x-1>=0 and y+1<mh then
		if map:get(x-1,y+1)>=value then tl=true end
	end
	
	if x+1<mw and y+1<mh then
		if map:get(x+1,y+1)>=value then tr=true end
	end
	
	-- Generate edge code
	local edge=0
	if t then edge=edge+1 end
	if l then edge=edge+2 end
	if r then edge=edge+4 end
	if b then edge=edge+8 end
	
	-- Generate corner codes
	local corners={false,false,false,false}
	if tl then
		if t==false and l==false then corners[1]=true end
	end
	
	if tr then
		if t==false and r==false then corners[2]=true end
	end
	
	if bl then
		if b==false and l==false then corners[3]=true end
	end
	
	if br then
		if b==false and r==false then corners[4]=true end
	end
	
	local corner=0
	if corners[1] then corner=corner+1 end
	if corners[2] then corner=corner+2 end
	if corners[3] then corner=corner+4 end
	if corners[4] then corner=corner+8 end
	
	return edge,corner+15
end
